/**
 * 
 */
package sf.controller;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;

import javax.faces.context.FacesContext;
import javax.faces.model.SelectItem;

/**
 * @author Zwaldi
 *
 */
public class LocaleList {
	/** singleton instance */
	private static LocaleList instance = new LocaleList();

	/** holds all availible locales */
	private List<SelectItem> supportedLocaleList = new LinkedList<SelectItem>();

	/**
	 * get the instance
	 * @return (<code>LocaleList</code>)
	 */
	public static LocaleList getSingleton() {
		return instance;
	}

	/**
	 * create the instance and initialize list
	 */
	LocaleList() {
		super();
		//save current locale setting of VM
		Locale old = FacesContext.getCurrentInstance().getViewRoot().getLocale();

		/*
		Locale[] availible = Locale.getAvailableLocales();
		System.out.println("All known Locales:");
		for (int i = 0; i < availible.length; i++) {
			//System.out.println(MessageFactory.getMessage(availible[i],,null));
			String country = availible[i].getCountry();
			if (!country.equals("")) {
				Locale.setDefault(availible[i]);
				String insertLabel = availible[i].getDisplayLanguage() + " (" + availible[i].getDisplayCountry() + ")";
				String insertId = availible[i].getLanguage() + "_" + availible[i].getCountry();
				SelectItem locElem = new SelectItem(insertId, insertLabel);
				supportedLocaleList.add(locElem);
			}
		}
		*/
		
		//get all supported locales
		Iterator locales = FacesContext.getCurrentInstance().getApplication().getSupportedLocales();
		
		while (locales.hasNext())
		{
			//get it locale
			Locale locale = (Locale) locales.next();
			//set locale to get language encoded Strings
			Locale.setDefault(locale);

			
			//get country code (de,gb,...) and name (greatbrit., deutschland)
			String country = locale.getCountry();
			String displayCountry = locale.getDisplayCountry();
			if (country != null && country.length() > 0) {
				country = "_" + country;
				displayCountry = " (" + displayCountry + ")";
			}
			//create the entry (id and label)
			SelectItem locElem = new SelectItem(
				//create the id; language_country code
				locale.getLanguage() + country,
				//and create the label: display langugae (display coutry)
				locale.getDisplayLanguage() + displayCountry
			);
				//insert element in list
			sortInsert(locElem, -1);
		}
		
		//restore locale setting of VM
		Locale.setDefault(old);
	}

	/**
	 * destructor deletes the locale list an finalize all entries 
	 */
	public void finalize() {
		supportedLocaleList = null;
		instance = null;
	}
	
	/**
	 * getter method for supportedLocaleList
	 * @return (<code>List<SelectItem></code>) supportedLocaleList
	 */
	public List<SelectItem> getSupportedLocaleList() {
		return supportedLocaleList;
	}

	/**
	 * get a item locale of a locale 
	 * @param locale (<code>Locale</code>) the locale to search
	 * @return
	 */
	public SelectItem getItem(Locale locale) {
		String country = locale.getCountry();
		if (country != null && country.length() > 0)
			country = "_" + country;
		//get all supported locales
		Iterator locales = supportedLocaleList.listIterator();
		//and iterate over them
		//String searchLocale =  + "_"
		while (locales.hasNext()) {
			//get a locale
			SelectItem locElem = (SelectItem) locales.next();
			//get  the id
			String locElemId = (String) locElem.getValue();
			//check for variant
			if ((locale.getLanguage() + country).equals(locElemId))
				return locElem;
		}

		//default return current
		return new SelectItem(locale.getDisplayName() + " (" + locale.getDisplayVariant() +")");
	}
	
	/**
	 * get the locale reffered by id 
	 * @param id (<code>int</code>) the id to search
	 * @return
	 */
	public Locale getLocale(int id) {
		//chekc if index is in range
		if (id > 0 && id < supportedLocaleList.size()) {
			String locElemId = (String) supportedLocaleList.get(id).getValue();
			Iterator locales = FacesContext.getCurrentInstance().getApplication().getSupportedLocales();
			while (locales.hasNext()) {
				Locale locale = (Locale) locales.next();
				if ((locale.getLanguage() + "_" + locale.getCountry()).equals(locElemId))
					return locale;
			}
		}
		
		//default return current
		return FacesContext.getCurrentInstance().getViewRoot().getLocale();
	}
	
	/**
	 * insert a element sorted by label
	 * @param insert (<code>SelectItem</code>) the element to insert
	 * @param sort (<code>int</code>) 1 -> descending ... -1 -> ascending
	 */
	private void sortInsert(SelectItem insert, int sort)
	{
		//search position to insert
		//search over all entries
		int pos;
		int size = supportedLocaleList.size();
		for (pos = 0; pos < size; pos++) {
			//if the value to insert is lower as the current
			if (insert.getLabel().compareTo(supportedLocaleList.get(pos).getLabel()) == sort)
				//exit for
				break;
		}
		supportedLocaleList.add(pos, insert);
	}
}
