/**
 * 
 */
package sf.controller;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;

import sf.modell.Subject;
import util.message.CleverMessageFactory;

/**
 * Verwaltet die Themenliste
 * 
 * @author Danny
 *
 */
public class SubjectController {
	public static final String LESS_SELECTION_ID = "subjectController_fail_lessSelection";
	public static final String LESS_SELECTION_TXT = "Entry selected!!!";
	public static final String INVALID_MOVEUPFIRST_ID = "subjectController_fail_moveUpFirst";
	public static final String INVALID_MOVEUPFIRST_TXT = "Move failed!!!";
	public static final String INVALID_SWAP_ID = "subjectController_fail_swap";
	public static final String INVALID_SWAP_TXT = "Swap failed!!!";
	public static final String INVALID_MOVEDOWNLAST_ID = "subjectController_fail_moveDownLast";
	public static final String INVALID_MOVEDOWNLAST_TXT = "Move failed!!!";
	
	/** neu // editieren */
	private static final String OUTCOME_EDIT = "ta_subject_edit";
	/** lschen */
	private static final String OUTCOME_LIST = "ta_subject";

	/** Anzahl an Eintrgen pro Seite */
	private static final int PER_PAGE = 25;
	
	/** Anzeigepostion */
	private int pos;
	
	private boolean hasBeforePage;
	private boolean hasNextPage;
	
	/** zu editierender Thema */
	private Subject currentSubject;
	
	/** enthlt alle teilnehmer fr die datatable */
	private List<SubjectRow> subjectRows = new ArrayList<SubjectRow>();
	
	private void simulateDB() {
		Subject s = new Subject(); s.setText("Virtualisierung von Betriebssystemen: Das VM Betriebssystem von IBM");
		SubjectRow r = new SubjectRow(); r.setSubject(s); subjectRows.add(r);
		s = new Subject(); s.setText("Virtualisierung von Betriebssystemen: Der Xen VM-Monitor");
		r = new SubjectRow(); r.setSubject(s); subjectRows.add(r);
		s = new Subject(); s.setText("Virtualisierung von Betriebssystemen: Die VMware Architektur");
		r = new SubjectRow(); r.setSubject(s); subjectRows.add(r);
		s = new Subject(); s.setText("Ajax");
		r = new SubjectRow(); r.setSubject(s); subjectRows.add(r);
		s = new Subject(); s.setText("C# 3.0");
		r = new SubjectRow(); r.setSubject(s); subjectRows.add(r);
		s = new Subject(); s.setText("Cocoon");
		r = new SubjectRow(); r.setSubject(s); subjectRows.add(r);
		s = new Subject(); s.setText("Grid Computing mit Linux");
		r = new SubjectRow(); r.setSubject(s); subjectRows.add(r);
		s = new Subject(); s.setText("Groovy on Grails");
		r = new SubjectRow(); r.setSubject(s); subjectRows.add(r);
		s = new Subject(); s.setText("Groovy: Scripting mit Java");
		r = new SubjectRow(); r.setSubject(s); subjectRows.add(r);
		s = new Subject(); s.setText("Java Server Faces");
		r = new SubjectRow(); r.setSubject(s); subjectRows.add(r);
		s = new Subject(); s.setText("Mashup mit Web 2.0");
		r = new SubjectRow(); r.setSubject(s); subjectRows.add(r);
		s = new Subject(); s.setText("Perl&nbsp;6, Parrot und Pugs");
		r = new SubjectRow(); r.setSubject(s); subjectRows.add(r);
		s = new Subject(); s.setText("Ruby On Rails");
		r = new SubjectRow(); r.setSubject(s); subjectRows.add(r);
		s = new Subject(); s.setText("Spring");
		r = new SubjectRow(); r.setSubject(s); subjectRows.add(r);
		s = new Subject(); s.setText("User Mode Linux");
		r = new SubjectRow(); r.setSubject(s); subjectRows.add(r);
		s = new Subject(); s.setText("Web-Frameworks mit Python");
		
		Collections.sort(subjectRows);
	}
	
	private void checkForFirstPage() {
		hasBeforePage = (pos < PER_PAGE);
	}
	
	private void checkForLastPage() {
		hasNextPage = (pos + PER_PAGE) > subjectRows.size();
	}
	
	public SubjectController() {
		simulateDB();
		
		pos = 0;
		checkForFirstPage();
		checkForLastPage();
	}
	
	/** 
	 * erstellt einen neuen (leeren) teilnehmer
	 */
	public String addSubject() {
		currentSubject = new Subject();
		return OUTCOME_EDIT;
	}
	
	/** 
	 * editiert (ersten) ausgewhlten teilnehmer der aktuellen seite
	 */
	public String editSubject() {
		//ausgewhlte ermitteln
		List<Integer> selected = getSelectedPosHelper(1);
		//einer ausgewhlt
		if (selected.size() == 1 && selected.get(0) != null)
			currentSubject = subjectRows.get(selected.get(0)).getSubject();
		else
			addSubject();
		
		return OUTCOME_EDIT;
	}

	/**
	 * Lscht alle ausgewhlten Thema
	 * @return
	 */
	public String update() {
		//testen ob neues thema eingetragfen wurde 
		// (nummer nicht in der liste...)
		boolean isNew = true;
		
		for (SubjectRow row : subjectRows)
			if (currentSubject.getNo().equals(row.getSubject().getNo())) {
				isNew = false;
				break;
			}
				
		if (isNew) {
			//also an die liste anfgen
			SubjectRow _new = new SubjectRow();
			_new.setSubject(currentSubject);
			subjectRows.add(_new);
		}
		
		return OUTCOME_LIST;
	}
	
	/**
	 * Lscht alle ausgewhlten Themen
	 * @return
	 */
	public String deleteSubjects() {
		boolean oneDeleted=false;
		
		for (int i=pos; i < Math.min(pos + PER_PAGE, subjectRows.size()) ; i++)
			//ausgewhlt?
			if (subjectRows.get(i).isSelected()) {
				oneDeleted=true;
				//dann lschen
				subjectRows.remove(i);
				//ein element weniger in der liste, zeiger runtersetzen
				i-=1;
			}

		//wenn keiner ausgewhlt war
		if (!oneDeleted)
			FacesContext.getCurrentInstance()
			.addMessage(null, 
				CleverMessageFactory.getMessage(LESS_SELECTION_ID, new Object[] {1, 0}, LESS_SELECTION_TXT)
			);
		
		return OUTCOME_LIST;
	}
	
	/**
	 * @return the pER_PAGE
	 */
	public int getPER_PAGE() {
		return PER_PAGE;
	}

	/**
	 * @param per_page the pER_PAGE to set
	public void setPER_PAGE(int per_page) {
	}
	 */

	/**
	 * @return the pos
	 */
	public int getPos() {
		return pos;
	}

	/**
	 * @param pos the pos to set
	 */
	public void setPos(int pos) {
		this.pos = pos;
	}
	
	public void backward(ActionEvent e) {
		pos -= PER_PAGE;
		if (pos < 0) 
			pos = 0;
		checkForFirstPage();
	}

	public void forward(ActionEvent e) {
		pos += PER_PAGE;
		if (pos > subjectRows.size())
			pos = subjectRows.size();
		checkForLastPage();
	}
	
	/**
	 * @return the subjectRows
	 */
	public List<SubjectRow> getSubjectRows() {
		return subjectRows;
	}

	/**
	 * @param subjectRows the subjectRows to set
	 */
	public void setSubjectRows(List<SubjectRow> subjectRows) {
		this.subjectRows = subjectRows;
	}

	/**
	 * @return the currentSubject
	 */
	public Subject getCurrentSubject() {
		return currentSubject;
	}

	/**
	 * @param currentSubject the currentSubject to set
	 */
	public void setCurrentSubject(Subject currentSubject) {
		this.currentSubject = currentSubject;
	}

	/**
	 * sucht selektierte eintrge
	 * @param count anzahl der zu suchenden selektierten
	 * @return List<Integer> maximal count (selektierte) eintrge 
	 */
	private List<Integer> getSelectedPosHelper(int count) {
		List<Integer> list = new ArrayList<Integer>(count);
		
		//durch die liste gehen
		for (int i=pos; i < Math.min(pos + PER_PAGE, subjectRows.size()) ; i++) {
			SubjectRow current = subjectRows.get(i);
			//markierte suchen
			if (current.isSelected()) {
				list.add(i);
			}
			if (list.size() >= count)
				break;
		}
		
		return list;
	}
	
	private void swapHelper(Subject s1, Subject s2) {
		int tmp = s1.getNo();
		s1.setNo(s2.getNo());
		s2.setNo(tmp);
	}
	
	/**
	 * vertauscht die nummern von 2 themen..
	 */
	public String swap() {
		List<Integer> selected = getSelectedPosHelper(2);	
		
		//2 gefunden -> numemrn tauschen
		if (selected.size() == 2) {
			swapHelper(
				subjectRows.get(selected.get(0)).getSubject(), 
				subjectRows.get(selected.get(1)).getSubject()
			);
			Collections.sort(subjectRows);
		}
		//zu wenig ausgewhlt
		else {
			FacesContext.getCurrentInstance().addMessage(null, 
				CleverMessageFactory.getMessage(
					LESS_SELECTION_ID, 
					new Object[] {2, selected.size()}, 
					LESS_SELECTION_TXT
				)
			);
		}
		return OUTCOME_LIST;
	}

	/**
	 * eintrag um eine nummer hoch schieben
	 */
	public String moveUp() {
		//ausgewhlte ermitteln
		List<Integer> selected = getSelectedPosHelper(1);
		//einer gefunden und nicht der erste
		if (selected.size() == 1 && selected.get(0) > pos) {
			//dann tauschen
			swapHelper( 
				subjectRows.get(selected.get(0)).getSubject(),
				subjectRows.get(selected.get(0) - 1).getSubject()
			);
			//und neu sortieren
			Collections.sort(subjectRows);
		}
		//sonst fehler
		else
			//wenn keiner ausgewhlt war
			if (selected.size() == 0)
				FacesContext.getCurrentInstance().addMessage(null, 
						CleverMessageFactory.getMessage(
							LESS_SELECTION_ID, 
							new Object[] {1, selected.size()}, 
							LESS_SELECTION_TXT
						)
					);
			else
				FacesContext.getCurrentInstance().addMessage(null, 
					CleverMessageFactory.getMessage(
						INVALID_MOVEUPFIRST_ID,
						subjectRows.get(selected.get(0)).getSubject().getText(),
						INVALID_MOVEUPFIRST_TXT
					)
				);
		
		return OUTCOME_LIST;
	}
	
	/**
	 * eintrag um eine nummer runter schieben
	 */
	public String moveDown() {
		//ausgewhlte ermitteln
		List<Integer> selected = getSelectedPosHelper(1);	
		if (selected.size() == 1 && selected.get(0) < Math.min(pos + PER_PAGE, subjectRows.size()) - 1) {
			swapHelper( 
				subjectRows.get(selected.get(0)).getSubject(),
				subjectRows.get(selected.get(0) + 1).getSubject()
			);
			Collections.sort(subjectRows);
			}
		else
			//wenn keiner ausgewhlt war
			if (selected.size() == 0)
				FacesContext.getCurrentInstance().addMessage(null, 
					CleverMessageFactory.getMessage(
						LESS_SELECTION_ID, 
						new Object[] {1, selected.size()}, 
						LESS_SELECTION_TXT
					)
				);
			//letzter ausgewhlt
			else
				FacesContext.getCurrentInstance().addMessage(null, 
					CleverMessageFactory.getMessage(
						INVALID_MOVEDOWNLAST_ID, 
						subjectRows.get(selected.get(0)).getSubject().getText(), 
						INVALID_MOVEDOWNLAST_TXT
					)
				);
		
		return OUTCOME_LIST;
	}
	
	/**
	 * eintrag an anfang schieben
	 */
	public String moveFirst() {
		//ausgewhlte ermitteln
		List<Integer> selected = getSelectedPosHelper(1);	
		//nicht der erste
		if (selected.size() == 1 && selected.get(0) > pos) {
			//erster rank
			subjectRows.get(selected.get(0)).getSubject().setNo(pos);
			//bei restlichen eintrgen rank erhhen
			for (int j=pos; j < selected.get(0); j++) {
				Subject s = subjectRows.get(j).getSubject();
				s.setNo(new Integer(s.getNo().intValue() + 1));
			}
			Collections.sort(subjectRows);
		}
		else
			//wenn keiner ausgewhlt war
			if (selected.size() == 0)
				FacesContext.getCurrentInstance().addMessage(null, 
						CleverMessageFactory.getMessage(
							LESS_SELECTION_ID, 
							new Object[] {1, selected.size()}, 
							LESS_SELECTION_TXT
						)
					);
			else
				FacesContext.getCurrentInstance().addMessage(null, 
					CleverMessageFactory.getMessage(
						INVALID_MOVEUPFIRST_ID,
						subjectRows.get(selected.get(0)).getSubject().getText(),
						INVALID_MOVEUPFIRST_TXT
					)
				);		
		return OUTCOME_LIST;
	}

	/**
	 * eintrag ans ende schieben
	 */
	public String moveLast() {
		//ausgewhlte ermitteln
		List<Integer> selected = getSelectedPosHelper(1);	
		//nicht der letzte
		if (selected.size() == 1 && selected.get(0) < Math.min(pos + PER_PAGE, subjectRows.size()) - 1) {
			subjectRows.get(selected.get(0)).getSubject().setNo(Math.min(pos + PER_PAGE, subjectRows.size()));
			//bei restlichen eintrgen rank erniedrigen
			for (int j=selected.get(0); j < pos; j++) {
				Subject s = subjectRows.get(j).getSubject();
				s.setNo(new Integer(s.getNo().intValue() - 1));
			}
			Collections.sort(subjectRows);
		}
		else
			//wenn keiner ausgewhlt war
			if (selected.size() == 0)
				FacesContext.getCurrentInstance().addMessage(null, 
					CleverMessageFactory.getMessage(
						LESS_SELECTION_ID, 
						new Object[] {1, selected.size()}, 
						LESS_SELECTION_TXT
					)
				);
			//letzter ausgewhlt
			else
				FacesContext.getCurrentInstance().addMessage(null, 
					CleverMessageFactory.getMessage(
						INVALID_MOVEDOWNLAST_ID, 
						subjectRows.get(selected.get(0)).getSubject().getText(), 
						INVALID_MOVEDOWNLAST_TXT
					)
				);
		
		return OUTCOME_LIST;
	}

	/**
	 * @return the hasBeforePage
	 */
	public boolean isHasBeforePage() {
		return hasBeforePage;
	}

	/**
	 * @param hasBeforePage the hasBeforePage to set
	 */
	public void setHasBeforePage(boolean hasBeforePage) {
		this.hasBeforePage = hasBeforePage;
	}

	/**
	 * @return the hasNextPage
	 */
	public boolean isHasNextPage() {
		return hasNextPage;
	}

	/**
	 * @param hasNextPage the hasNextPage to set
	 */
	public void setHasNextPage(boolean hasNextPage) {
		this.hasNextPage = hasNextPage;
	}
}
