/**
 * 
 */
package sf.controller;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;
import javax.faces.model.SelectItem;

import sf.modell.Member;
import sf.modell.Subject;
import sf.modell.Tutorial;
import util.message.CleverMessageFactory;

/**
 * Verwaltet die TeilnehmerListe
 * 
 * @author Danny
 *
 */
public class TutorialController {
	/** neu // editieren */
	private static final String OUTCOME_EDIT = "ta_tutorial_edit";
	/** lschen */
	private static final String OUTCOME_LIST = "ta_tutorial";

	/** Anzahl an Eintrgen pro Seite */
	private static final int PER_PAGE = 20;

	/** Name des Fehlertextes fr Teilnehmer der schon ein Seminar hlt */
	public static final String DUPLICATE_MEMBER_MSG_ID = "validation_TutorialController_duplicateMember";
	/** Fehlertext fr Teilnehmer der schon ein Seminar hlt */
	public static final String DUPLICATE_MEMBER_MSG_TXT = "The member {0} is already in tutorial {1} !!!";
	/** Name des Fehlertextes fr fr doppel vergebenes Seminar */
	public static final String DUPLICATE_TUTORIAL_MSG_ID = "validation_TutorialController_duplicateTutorial";
	/** Fehlertext fr Teilnehmer der schon ein Seminar hlt */
	public static final String DUPLICATE_TUTORIAL_MSG_TXT = "The tutorial {0} is already assigned with {1} !!!";
	
	/** Anzeigepostion */
	private int pos;
	
	private boolean notFirstPage;
	private boolean notLastPage;
	
	/** zu editierender Teilnehmer */
	private Tutorial currentTutorial;
	/** just a hack *** keine ids vorhanden - daher notwendig um edit und new zu unterscheiden*/
	private boolean isNew;
	
	/** enthlt alle vortge fr die datatable */
	private List<TutorialRow> tutorialRows = new ArrayList<TutorialRow>();

	private List<SelectItem> subjectList;
	private List<SelectItem> memberList; 
	
	private void simulateDB() {

		memberList = new LinkedList<SelectItem>();
		List<MemberRow> memberRows = (List<MemberRow>) FacesContext.getCurrentInstance().getApplication().createValueBinding("#{MemberControllerBean.memberRows}").getValue(FacesContext.getCurrentInstance());
		for (MemberRow mR : memberRows) {
			Member m = mR.getMember();
			SelectItem sI = new SelectItem(m.getName(), m.getName() + "," + mR.getMember().getFirstName());
			memberList.add(sI);
		}
			
		subjectList = new LinkedList<SelectItem>();
		List<SubjectRow> subjectRows = (List<SubjectRow>) FacesContext.getCurrentInstance().getApplication().createValueBinding("#{SubjectControllerBean.subjectRows}").getValue(FacesContext.getCurrentInstance());
		for (SubjectRow sR : subjectRows) {
			Subject s = sR.getSubject();
			SelectItem sI = new SelectItem(s.getText());
			subjectList.add(sI);
		}
			
		Tutorial t = new Tutorial(); t.setSubject(subjectRows.get(9).getSubject());	t.setMember(memberRows.get(3).getMember()); t.setDate(new Date(106,11,1,12,30));
		TutorialRow tr = new TutorialRow(); tr.setTutorial(t);
		tutorialRows.add(tr);
		t = new Tutorial(); t.setSubject(subjectRows.get(6).getSubject());	t.setMember(memberRows.get(4).getMember()); t.setDate(new Date(106,10,14,12,30));
		tr = new TutorialRow(); tr.setTutorial(t);
		tutorialRows.add(tr);
		t = new Tutorial(); t.setSubject(subjectRows.get(3).getSubject());	t.setMember(memberRows.get(1).getMember()); t.setDate(new Date(106,11,5,12,30));
		tr = new TutorialRow(); tr.setTutorial(t);
		tutorialRows.add(tr);

		Collections.sort(tutorialRows);
	}
	
	private void checkFirstPage() {
		notFirstPage = pos >= PER_PAGE;
	}
	
	private void checkLastPage() {
		notLastPage = (pos + PER_PAGE) <= tutorialRows.size();
	}
	
	public TutorialController() {
		simulateDB();
		
		pos = 0;
		checkFirstPage();
		checkLastPage();
	}
	
	/** 
	 * erstellt einen neuen (leeren) teilnehmer
	 */
	public String addTutorial() {
		isNew = true;
		currentTutorial = new Tutorial();
		currentTutorial.setMember(new Member());
		currentTutorial.setSubject(new Subject());
		return OUTCOME_EDIT;
	}
	
	/** 
	 * editiert (ersten) ausgewhlten teilnehmer der aktuellen seite
	 */
	public String editTutorial() {
		isNew = false;
		currentTutorial = null;
		for (int i=pos; i < Math.min(pos + PER_PAGE, tutorialRows.size()) ; i++)
			if (tutorialRows.get(i).isSelected()) {
				currentTutorial = tutorialRows.get(i).getTutorial(); 
				break;
			}
		if (currentTutorial == null)
			addTutorial();
		
		return OUTCOME_EDIT;
	}

	/**
	 * Aktualisiert einen Teilnehmer
	 * @return
	 */
	public String update() {
		//ist er neu dann anhngen
		if (isNew) {
			//also an die liste anfgen
			TutorialRow _new = new TutorialRow();
			_new.setTutorial(currentTutorial);
			tutorialRows.add(_new);
		}
		
		//neu sortieren
		Collections.sort(tutorialRows);
		
		return OUTCOME_LIST;
	}
	
	/**
	 * Lscht alle ausgewhlten Teilnehmer
	 * @return
	 */
	public String deleteTutorials() {
		//von anfang der aktuellen seite bis seitenende bzw. datenende suchen
		for (int i=pos; i < Math.min(pos + PER_PAGE, tutorialRows.size()) ; i++)
			//ausgewhlt?
			if (tutorialRows.get(i).isSelected()) {
				//dann lschen
				tutorialRows.remove(i);
				//ein element weniger in der liste, zeiger runtersetzen
				i-=1;
			}
		
		return OUTCOME_LIST;
	}
	
	/**
	 * @return the pER_PAGE
	 */
	public int getPER_PAGE() {
		return PER_PAGE;
	}

	/**
	 * @param per_page the pER_PAGE to set
	public void setPER_PAGE(int per_page) {
	}
	 */

	/**
	 * @return the pos
	 */
	public int getPos() {
		return pos;
	}

	/**
	 * @param pos the pos to set
	 */
	public void setPos(int pos) {
		this.pos = pos;
	}
	
	public void backward(ActionEvent e) {
		pos -= PER_PAGE;
		if (pos < 0) 
			pos = 0;
		checkFirstPage();
	}

	public void topage(ActionEvent e) {
		int pageNo = Integer.parseInt(((String) e.getComponent().findComponent("abc").getAttributes().get("page")));
		pos = pageNo * PER_PAGE; 
		checkFirstPage();
		checkLastPage();
	}

	public void forward(ActionEvent e) {
		pos += PER_PAGE;
		if (pos > tutorialRows.size())
			pos = tutorialRows.size();
		checkLastPage();
	}
	
	/**
	 * @return the notFirstPage
	 */
	public boolean isNotFirstPage() {
		checkFirstPage();
		return notFirstPage;
	}

	/**
	 * @param notFirstPage the notFirstPage to set
	 */
	public void setNotFirstPage(boolean notFirstPage) {
		this.notFirstPage = notFirstPage;
	}

	/**
	 * @return the notLastPage
	 */
	public boolean isNotLastPage() {
		checkLastPage();
		return notLastPage;
	}

	/**
	 * @param notLast the notLast to set
	 */
	public void setNotLastPage(boolean notLastPage) {
		this.notLastPage = notLastPage;
	}

	/**
	 * @return the currentTutorial
	 */
	public Tutorial getCurrentTutorial() {
		return currentTutorial;
	}

	/**
	 * @param currentTutorial the currentTutorial to set
	 */
	public void setCurrentTutorial(Tutorial currentTutorial) {
		this.currentTutorial = currentTutorial;
	}

	public List<SelectItem> getMemberList() {
		return memberList;
	}

	public void setMemberList(List<SelectItem> memberList) {
		this.memberList = memberList;
	}

	public List<SelectItem> getSubjectList() {
		return subjectList;
	}

	public void setSubjectList(List<SelectItem> subjectList) {
		this.subjectList = subjectList;
	}

	public List<TutorialRow> getTutorialRows() {
		return tutorialRows;
	}

	public void setTutorialRows(List<TutorialRow> tutorialRows) {
		this.tutorialRows = tutorialRows;
	}

	/**
	 * upodates current turotial after selecting a new value
	 * 
	 * @param vce
	 */
	public void handleValueChange(ValueChangeEvent vce) {
		if (vce.getComponent().getId().equals("member")) {
			List<MemberRow> memberRows = (List<MemberRow>) FacesContext.getCurrentInstance().getApplication().createValueBinding("#{MemberControllerBean.memberRows}").getValue(FacesContext.getCurrentInstance());
			for (MemberRow mR : memberRows)
				if (mR.getMember().getName().equals(vce.getNewValue()))
					currentTutorial.setMember(mR.getMember());
		}
		if (vce.getComponent().getId().equals("subject")) {
			List<SubjectRow> subjectRows = (List<SubjectRow>) FacesContext.getCurrentInstance().getApplication().createValueBinding("#{SubjectControllerBean.subjectRows}").getValue(FacesContext.getCurrentInstance());
			for (SubjectRow sR : subjectRows)
				if (sR.getSubject().getText().equals(vce.getNewValue()))
					currentTutorial.setSubject(sR.getSubject());
		}
	}
	/**
	 * berprft ob ein teilnehmer nicht schon in anderem semniar eingetragen ist
	 * @param context
	 * @param toValidate
	 * @param value
	 */
	 public void validateMemberSelection(FacesContext context, 
             UIComponent toValidate,
             Object value) {
		 int maxHits = isNew ? 0 : 1;
		 int hits = 0;
		 // bei neu kein vorkommen erlaubt, bei edit maximal eins
		 for (TutorialRow tR : tutorialRows) {
			 Tutorial t = tR.getTutorial();
			 if (t.getMember().getName().equals((String) value))
				if (hits < maxHits)
					hits++;
				else {
					 ((UIInput) toValidate).setValid(false);
					 context.addMessage("f:member",
						CleverMessageFactory.getMessage(
								DUPLICATE_MEMBER_MSG_ID,
								new Object[] {t.getMember().getName() + ", " + t.getMember().getFirstName(), t.getSubject().getText()},
								DUPLICATE_MEMBER_MSG_TXT)
							);
			 }
		 }
	 }

		/**
		 * berprfen ob seminar nicht schon von einem anderen teilnehemr gehalten wird und 
		 * @param context
		 * @param toValidate
		 * @param value
		 */
		 public void validateSubjectSelection(FacesContext context, 
	             UIComponent toValidate,
	             Object value) {
			 int maxHits = isNew ? 0 : 1;
			 int hits = 0;
			 // bei neu kein vorkommen erlaubt, bei edit maximal eins
			 for (TutorialRow tR : tutorialRows) {
				Tutorial t = tR.getTutorial();
				if 	(t.getSubject().getText().equals((String) value)) 
					if (hits < maxHits)
						hits++;
					else {
						 ((UIInput) toValidate).setValid(false);
						 context.addMessage("f:subject",
							CleverMessageFactory.getMessage(
								DUPLICATE_TUTORIAL_MSG_ID,
								new Object[] {t.getSubject().getText(), t.getMember().getName() + ", " + t.getMember().getFirstName()},
								DUPLICATE_TUTORIAL_MSG_TXT)
							);
					 }
			 }

		 }

}
